# Generated by Django 5.2.6 on 2025-12-09 14:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_license', '0005_alter_license_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='license',
            name='support_status',
            field=models.CharField(default='unknown', max_length=20),
        ),
        migrations.AlterField(
            model_name='license',
            name='status',
            field=models.CharField(default='active', max_length=20),
        ),
    ]



from django.db.models import Sum
from django_tables2 import tables, TemplateColumn
from netbox.tables import NetBoxTable
from django.urls import reverse
from django.utils.html import format_html
from ..models import License, LicenseAssignment, LicenseType
from ..template_content import LICENSE_EXPIRY_PROGRESSBAR_TABLE

# ---------- LicenseType ----------

class LicenseTypeTable(NetBoxTable):
    name = tables.Column(linkify=True)
    slug = tables.Column()
    manufacturer = tables.Column(verbose_name="Manufacturer", linkify=True)

    instances = tables.Column(
        verbose_name="Instances",
        accessor="license_count",
        order_by="license_count",
        empty_values=(),
    )

    def render_instances(self, value, record):
        url = reverse('plugins:netbox_license:license_list') + f'?license_type_id={record.pk}'
        return format_html('<a href="{}">{}</a>', url, value)

    base_license = tables.Column(
        accessor='base_license',
        verbose_name='Base License',
        linkify=True
    )
    product_code = tables.Column(verbose_name="Product Code")
    ean_code = tables.Column(verbose_name="EAN Code")
    volume_type = tables.Column(verbose_name="Volume Type")
    volume_relation = tables.Column(verbose_name="Volume Relation")
    license_model = tables.Column(verbose_name="License Model")
    purchase_model = tables.Column(verbose_name="Purchase Model")
    description = tables.Column()

    class Meta(NetBoxTable.Meta):
        model = LicenseType
        fields = (
            "id", "name", "slug", "manufacturer","base_license",
            "product_code", "ean_code",
            "volume_type", "volume_relation", "license_model", "purchase_model",
            "description"
        )
        default_columns = (
            "name", "manufacturer","base_license",
            "product_code", "volume_type",
            "license_model", "instances",
        )



# from netbox.jobs import JobRunner, system_job
# from core.choices import JobIntervalChoices
# from netbox_license.models.license import License

#### !!!!!!!!!!! # This job is currently disabled because it currently updates the status, but does not trigger the required event. Use script instead.

# @system_job(interval=JobIntervalChoices.INTERVAL_MINUTELY)
# class LicenseStatusCheckJob(JobRunner):
#     """
#     Job to verify and update license statuses and runs every minute.
#     """

#     class Meta:
#         name = "License Status Checker"

#     def run(self, *args, **kwargs):
#         self.logger.info("Starting License Status Check job...")

#         try:
#             licenses = License.objects.all()
#             self.logger.info(f"Found {licenses.count()} licenses to check.")
#             updated_count = 0
#             for license in licenses:
#                 new_status = license.compute_status()
#                 if license.status != new_status:
#                     self.logger.debug(
#                         f"License {license.pk}: status changed from {license.status} to {new_status}"
#                     )
#                     license.status = new_status
#                     license.save()
#                     updated_count += 1

#             self.logger.info(f"License Status Check completed. Updated {updated_count} licenses.")

#         except Exception as e:
#             # Log the error and re-raise so NetBox marks the job as failed
#             self.logger.error(f"Error during License Status Check: {repr(e)}")
#             raise

# ###### Info:
# # class JobIntervalChoices(ChoiceSet):
# #     INTERVAL_MINUTELY = 1
# #     INTERVAL_HOURLY = 60
# #     INTERVAL_DAILY = 60 * 24
# #     INTERVAL_WEEKLY = 60 * 24 * 7
# #     CHOICES = (
# #         (INTERVAL_MINUTELY, _('Minutely')),
# #         (INTERVAL_HOURLY, _('Hourly')),
# #         (INTERVAL_HOURLY * 12, _('12 hours')),
# #         (INTERVAL_DAILY, _('Daily')),
# #         (INTERVAL_WEEKLY, _('Weekly')),
# #         (INTERVAL_DAILY * 30, _('30 days')),
# #     )



from netbox.api.viewsets import NetBoxModelViewSet
from netbox_license.models import License, LicenseAssignment, LicenseType
from .serializers import LicenseTypeSerializer, LicenseSerializer, LicenseAssignmentSerializer
from netbox_license.filtersets import licensetypes, LicenseFilterSet, licenseassignments


class LicenseTypeViewSet(NetBoxModelViewSet):
    """API viewset for managing License Types"""
    queryset = LicenseType.objects.all()
    serializer_class = LicenseTypeSerializer
    filterset_class = licensetypes.LicenseTypeFilterSet

class LicenseViewSet(NetBoxModelViewSet):
    """API view for managing Licenses"""
    queryset = License.objects.all()
    serializer_class = LicenseSerializer
    filterset_class = LicenseFilterSet

class LicenseAssignmentViewSet(NetBoxModelViewSet):
    """API viewset for managing LicenseAssignments"""
    queryset = LicenseAssignment.objects.all()
    serializer_class = LicenseAssignmentSerializer
    filterset_class = licenseassignments.LicenseAssignmentFilterSet




from django.db.models import Sum
from django_tables2 import tables, TemplateColumn
from netbox.tables import NetBoxTable
from django.urls import reverse
from django.utils.html import format_html
from ..models import LicenseAssignment
from ..template_content import LICENSE_EXPIRY_PROGRESSBAR_TABLE

# ---------- Assignments ----------

class LicenseAssignmentTable(NetBoxTable):
    
    license_key = tables.Column(
        accessor="license",
        verbose_name="License Key",
        linkify=True
    )
    license_type = tables.Column(
        accessor='license.license_type',
        verbose_name='License Type',
        linkify=True,
        order_by='license__license_type__name'
    )
    manufacturer = tables.Column(
        accessor="license.license_type.manufacturer",
        verbose_name="License Manufacturer",
        linkify=True
    )

    device = tables.Column(
        accessor="device",
        verbose_name="Device",
        linkify=True
    )
    device_type = tables.Column(
        accessor="device.device_type",
        verbose_name="Device Type",
        linkify=True
    )
    device_manufacturer = tables.Column(
        accessor="device.device_type.manufacturer",
        verbose_name="Device Manufacturer",
        linkify=True
    )
    virtual_machine = tables.Column(
        accessor="virtual_machine",
        verbose_name="Virtual Machine",
        linkify=True
    )
    volume = tables.Column(verbose_name="Volume")

    volume_relation = tables.Column(
        accessor="license.license_type.volume_relation",
        verbose_name="Volume Relation",
        order_by="license__license_type__volume_relation"
    )
    assigned_on = tables.Column(verbose_name="Assigned On")
    description = tables.Column(verbose_name="Description")

    class Meta(NetBoxTable.Meta):
        model = LicenseAssignment
        fields = (
            "license_key", "license_type", "manufacturer",
            "device", "device_type", "device_manufacturer",
            "virtual_machine", "volume", "volume_relation",
            "assigned_on", "description"
        )
        default_columns = (
            "license", "device", "virtual_machine", "volume",
        )
            
